import 'package:test/test.dart';

import 'package:moving_average/moving_average.dart';

void main() {
  group('Mean (average) tests', () {
    test('Mean of [] is NaN', () {
      final List<num> input = <num>[];
      final num result = mean(input);
      expect(result, isNaN);
    });

    test('Mean of [1] is 1', () {
      final List<num> input = <num>[1];
      const num expected = 1.0;
      final num result = mean(input);
      expect(result, expected);
    });

    test('Mean of [1..5] is 3.0', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      const num expected = 3.0;
      final num result = mean(input);
      expect(result, expected);
    });
  });

  group('Weighted Mean tests', () {
    test('Weighted Mean of [] is NaN', () {
      final List<num> input = <num>[];
      final num result = weightedMean(input);
      expect(result, isNaN);
    });

    test('Weighted Mean of [1..5] is 3.6666666666666665', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      const num expected = 3.6666666666666665;
      final num result = weightedMean(input);
      expect(result, expected);
    });
  });

  group('Sliding Window tests', () {
    test('Window of [], size 2', () {
      final List<num> input = <num>[];
      final List<List<num>> expected = <List<num>>[];
      final List<List<num>> result =
          window(input, 2, includePartial: false).toList(growable: false);
      expect(result, expected);
    });

    test('Window of [1], size 2, no partial', () {
      final List<num> input = <num>[1];
      final List<List<num>> expected = <List<num>>[];
      final List<List<num>> result =
          window(input, 2, includePartial: false).toList(growable: false);
      expect(result, expected);
    });

    test('Window of [1], size 2, partial', () {
      final List<num> input = <num>[1];
      final List<List<num>> expected = <List<num>>[
        <num>[1]
      ];
      final List<List<num>> result =
          window(input, 2, includePartial: true).toList(growable: false);
      expect(result, expected);
    });

    test('Window of [1..5], size 2, no partial', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      final List<List<num>> expected = <List<num>>[
        <num>[1, 2],
        <num>[2, 3],
        <num>[3, 4],
        <num>[4, 5]
      ];
      final List<List<num>> result =
          window(input, 2, includePartial: false).toList(growable: false);
      expect(result, expected);
    });

    test('Window of [1..5], size 2, partial', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      final List<List<num>> expected = <List<num>>[
        <num>[1],
        <num>[1, 2],
        <num>[2, 3],
        <num>[3, 4],
        <num>[4, 5]
      ];
      final List<List<num>> result =
          window(input, 2, includePartial: true).toList(growable: false);
      expect(result, expected);
    });
  });

  group('Moving Average (mean) tests', () {
    test('Moving Average of [], size 2, no partial is []', () {
      final List<num> input = <num>[];
      final List<num> expected = <num>[];
      final List<num> result = movingAverage(input, 2, includePartial: false);
      expect(result, expected);
    });

    test('Moving Average of [1..5], size 2, no partial', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      final List<num> expected = <num>[1.5, 2.5, 3.5, 4.5];
      final List<num> result = movingAverage(input, 2, includePartial: false);
      expect(result, expected);
    });
  });

  group('Weighted Moving Average tests', () {
    test('Weighted Moving Average of [], size 2, no partial is []', () {
      final List<num> input = <num>[];
      final List<num> expected = <num>[];
      final List<num> result =
          weightedMovingAverage(input, 2, includePartial: false);
      expect(result, expected);
    });

    test('Weighted Moving Average of [1], size 2, no partial', () {
      final List<num> input = <num>[1];
      final List<num> expected = <num>[];
      final List<num> result =
          weightedMovingAverage(input, 2, includePartial: false);
      expect(result, expected);
    });

    test('Weighted Moving Average of [1], size 2, partial', () {
      final List<num> input = <num>[1];
      final List<num> expected = <num>[1];
      final List<num> result =
          weightedMovingAverage(input, 2, includePartial: true);
      expect(result, expected);
    });

    test('Weighted Moving Average of [1..5], size 2, no partial', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      final List<num> expected = <num>[
        1.6666666666666667,
        2.6666666666666665,
        3.6666666666666665,
        4.666666666666667
      ];
      final List<num> result =
          weightedMovingAverage(input, 2, includePartial: false);
      expect(result, expected);
    });

    test('Weighted Moving Average of [1..5], size 2, partial', () {
      final List<num> input = <num>[1, 2, 3, 4, 5];
      final List<num> expected = <num>[
        1.0,
        1.6666666666666667,
        2.6666666666666665,
        3.6666666666666665,
        4.666666666666667
      ];
      final List<num> result =
          weightedMovingAverage(input, 2, includePartial: true);
      expect(result, expected);
    });
  });
}
